@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2005 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2005 Apple Inc. All rights reserved.
  * Copyright (c) 2005-2006 Paul Guyot <pguyot@kallisys.net>,
  * All rights reserved.
  *
- * $Id: darwintrace.c 30381 2007-10-26 03:39:02Z epimenov@macports.org $
+ * $Id: darwintrace.c 80129 2011-07-05 03:53:54Z jmr@macports.org $
  *
  * @APPLE_BSD_LICENSE_HEADER_START@
  * 
@@ -16,7 +16,7 @@
  * 2.  Redistributions in binary form must reproduce the above copyright
  *     notice, this list of conditions and the following disclaimer in the
  *     documentation and/or other materials provided with the distribution. 
- * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
+ * 3.  Neither the name of Apple Inc. ("Apple") nor the names of
  *     its contributors may be used to endorse or promote products derived
  *     from this software without specific prior written permission. 
  * 
@@ -38,6 +38,19 @@
 #include <config.h>
 #endif
 
+#if HAVE_SYS_CDEFS_H
+#include <sys/cdefs.h>
+#endif
+#if defined(_DARWIN_FEATURE_64_BIT_INODE) && !defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE)
+/* The architecture we're building for has multiple versions of stat.
+   We need to undo sys/cdefs.h changes for _DARWIN_FEATURE_64_BIT_INODE */
+#undef  __DARWIN_64_BIT_INO_T
+#define __DARWIN_64_BIT_INO_T 0
+#undef  __DARWIN_SUF_64_BIT_INO_T
+#define __DARWIN_SUF_64_BIT_INO_T ""
+#undef _DARWIN_FEATURE_64_BIT_INODE
+#endif
+
 #ifdef HAVE_CRT_EXTERNS_H
 #include <crt_externs.h>
 #endif
@@ -150,15 +163,24 @@ static char* __env_darwintrace_log;
 
 #if __STDC_VERSION__>=199901L
 #if DARWINTRACE_DEBUG_OUTPUT
-#define dprintf(...) fprintf(stderr, __VA_ARGS__)
+#define debug_printf(...) fprintf(stderr, __VA_ARGS__)
 #else
-#define dprintf(...)
+#define debug_printf(...)
 #endif
 #else
 #if DARWINTRACE_DEBUG_OUTPUT
-#define dprintf(format, param) fprintf(stderr, format, param)
+__attribute__ ((format (printf, 1, 2)))
+static inline
+int debug_printf(const char *format, ...) {
+    int ret;
+    va_list args;
+    va_start(args, format);
+    ret = vfprintf(stderr, format, args);
+    va_end(args);
+    return ret;
+}
 #else
-#define dprintf(format, param)
+#define debug_printf(format, param)
 #endif
 #endif
 
@@ -239,8 +261,10 @@ inline char* __darwintrace_alloc_env(const char* varName, const char* varValue)
 	if (varValue) {
 		int theSize = strlen(varName) + strlen(varValue) + 2;
 		theResult = (char*) malloc(theSize);
-		sprintf(theResult, "%s=%s", varName, varValue);
-		theResult[theSize - 1] = 0;
+		if (theResult) {
+		    snprintf(theResult, theSize, "%s=%s", varName, varValue);
+		    theResult[theSize - 1] = 0;
+		}
 	}
 	
 	return theResult;
@@ -250,6 +274,7 @@ inline char* __darwintrace_alloc_env(const char* varName, const char* varValue)
  * This function checks that envp contains the global variables we had when the
  * library was loaded and modifies it if it doesn't.
  */
+__attribute__((always_inline))
 inline char* const* __darwintrace_restore_env(char* const envp[]) {
 	/* allocate the strings. */
 	/* we don't care about the leak here because we're going to call execve,
@@ -320,10 +345,11 @@ inline char* const* __darwintrace_restore_env(char* const envp[]) {
 static void ask_for_filemap()
 {
 	filemap=exchange_with_port("filemap\t", sizeof("filemap\t"), 1, 0);
-	if((int)filemap==-1)
+	if(filemap==(char*)-1)
 		filemap=0;
 }
 
+__attribute__((always_inline))
 inline void __darwintrace_setup() {
 #define open(x,y,z) syscall(SYS_open, (x), (y), (z))
 #define close(x) syscall(SYS_close, (x))
@@ -333,13 +359,16 @@ inline void __darwintrace_setup() {
 			int sock=socket(AF_UNIX, SOCK_STREAM, 0);
 			struct sockaddr_un sun;
 			sun.sun_family=AF_UNIX;
-			strcpy(sun.sun_path, __env_darwintrace_log);
+			strncpy(sun.sun_path, __env_darwintrace_log, sizeof(sun.sun_path));
 			if(connect(sock, (struct sockaddr*)&sun, strlen(__env_darwintrace_log)+1+sizeof(sun.sun_family))!=-1)
 			{
-				dprintf("darwintrace: connect successful. socket %d\n", sock);
+				debug_printf("darwintrace: connect successful. socket %d\n", sock);
 				__darwintrace_fd=sock;
 				ask_for_filemap();
-			}else dprintf("connect failed %d :-(\n", errno);
+			} else {
+				debug_printf("connect failed: %s\n", strerror(errno));
+				abort();
+			}
 			errno = olderrno;
 		}
 	}
@@ -352,6 +381,7 @@ inline void __darwintrace_setup() {
  * path:		the path of the file
  * fd:			a fd to the file, or 0 if we don't have any.
  */
+__attribute__((always_inline))
 inline void __darwintrace_log_op(const char* op, const char* path, int fd) {
 	int size;
 	char somepath[MAXPATHLEN];
@@ -439,7 +469,7 @@ inline void __darwintrace_cleanup_path(char *path) {
     }
   }
 
-  dprintf("darwintrace: cleanup resulted in %s\n", path);
+  debug_printf("darwintrace: cleanup resulted in %s\n", path);
 }
 
 /*
@@ -470,10 +500,10 @@ static int ask_for_dependency(char * path)
 	if(is_directory(path))
 		return 1;
 	
-	strcpy(buffer, "dep_check\t");
-	strcpy(buffer+10, path);
+	strncpy(buffer, "dep_check\t", sizeof(buffer));
+	strncpy(buffer+10, path, sizeof(buffer)-10);
 	p=exchange_with_port(buffer, strlen(buffer)+1, 1, 0);
-	if((int)p==-1||!p)
+	if(p==(char*)-1||!p)
 		return 0;
 	
 	if(*p=='+')
@@ -528,6 +558,7 @@ static char * exchange_with_port(const char * buf, size_t len, int answer, char
 /*
  * return 1 if path (once normalized) is in sandbox or redirected, 0 otherwise.
  */
+__attribute__((always_inline))
 inline int __darwintrace_is_in_sandbox(const char* path, char * newpath) {
 	char * t, * p, * _;
 	int result=-1;
@@ -541,12 +572,14 @@ inline int __darwintrace_is_in_sandbox(const char* path, char * newpath) {
 		p=strdup(path);
 	else
 	{
-		p=(char*)malloc(BUFFER_SIZE);
-		(void) getcwd(p, BUFFER_SIZE-1);
-		_=p+strlen(p)+1;
-		if(_[-1]!='/')
-			*_++='/';
-		strncpy(_, path, BUFFER_SIZE-(_-p));
+		p=(char*)malloc(MAXPATHLEN);
+		if (getcwd(p, MAXPATHLEN-1) == NULL) {
+			fprintf(stderr, "darwintrace: getcwd: %s, path was: %s\n", strerror(errno), path);
+			abort();
+		}
+		if (p[strlen(p)-1] != '/')
+			strcat(p, "/");
+		strcat(p, path);
 	}
 	__darwintrace_cleanup_path(p);
 			
@@ -630,7 +663,7 @@ int open(const char* path, int flags, ...) {
 		__darwintrace_setup();
 		isInSandbox = __darwintrace_is_in_sandbox(path, newpath);
 		if (isInSandbox == 0) {
-			dprintf("darwintrace: creation/writing was forbidden at %s\n", path);
+			debug_printf("darwintrace: creation/writing was forbidden at %s\n", path);
 			errno = EACCES;
 			result = -1;
 		}
@@ -776,10 +809,10 @@ int unlink(const char* path) {
 	int result = 0;
 	int isInSandbox = __darwintrace_is_in_sandbox(path, 0);
 	if (isInSandbox == 1) {
-		dprintf("darwintrace: unlink was allowed at %s\n", path);
+		debug_printf("darwintrace: unlink was allowed at %s\n", path);
 	} else if (isInSandbox == 0) {
 		/* outside sandbox, but sandbox is defined: forbid */
-		dprintf("darwintrace: unlink was forbidden at %s\n", path);
+		debug_printf("darwintrace: unlink was forbidden at %s\n", path);
 		errno = EACCES;
 		result = -1;
 	}
@@ -798,7 +831,7 @@ int mkdir(const char* path, mode_t mode) {
 	int result = 0;
 	int isInSandbox = __darwintrace_is_in_sandbox(path, 0);
 	if (isInSandbox == 1) {
-		dprintf("darwintrace: mkdir was allowed at %s\n", path);
+		debug_printf("darwintrace: mkdir was allowed at %s\n", path);
 	} else if (isInSandbox == 0) {
 		/* outside sandbox, but sandbox is defined: forbid */
 		/* only consider directories that do not exist. */
@@ -807,7 +840,7 @@ int mkdir(const char* path, mode_t mode) {
 		err = lstat(path, &theInfo);
 		if ((err == -1) && (errno == ENOENT))
 		{
-			dprintf("darwintrace: mkdir was forbidden at %s\n", path);
+			debug_printf("darwintrace: mkdir was forbidden at %s\n", path);
 			errno = EACCES;
 			result = -1;
 		} /* otherwise, mkdir will do nothing (directory exists) or fail
@@ -828,10 +861,10 @@ int rmdir(const char* path) {
 	int result = 0;
 	int isInSandbox = __darwintrace_is_in_sandbox(path, 0);
 	if (isInSandbox == 1) {
-		dprintf("darwintrace: rmdir was allowed at %s\n", path);
+		debug_printf("darwintrace: rmdir was allowed at %s\n", path);
 	} else if (isInSandbox == 0) {
 		/* outside sandbox, but sandbox is defined: forbid */
-		dprintf("darwintrace: removing directory %s was forbidden\n", path);
+		debug_printf("darwintrace: removing directory %s was forbidden\n", path);
 		errno = EACCES;
 		result = -1;
 	}
@@ -850,10 +883,10 @@ int rename(const char* from, const char* to) {
 	int result = 0;
 	int isInSandbox = __darwintrace_is_in_sandbox(from, 0);
 	if (isInSandbox == 1) {
-		dprintf("darwintrace: rename was allowed at %s\n", from);
+		debug_printf("darwintrace: rename was allowed at %s\n", from);
 	} else if (isInSandbox == 0) {
 		/* outside sandbox, but sandbox is defined: forbid */
-		dprintf("darwintrace: renaming from %s was forbidden\n", from);
+		debug_printf("darwintrace: renaming from %s was forbidden\n", from);
 		errno = EACCES;
 		result = -1;
 	}
@@ -861,10 +894,10 @@ int rename(const char* from, const char* to) {
 	if (result == 0) {
 		isInSandbox = __darwintrace_is_in_sandbox(to, 0);
 		if (isInSandbox == 1) {
-			dprintf("darwintrace: rename was allowed at %s\n", to);
+			debug_printf("darwintrace: rename was allowed at %s\n", to);
 		} else if (isInSandbox == 0) {
 			/* outside sandbox, but sandbox is defined: forbid */
-			dprintf("darwintrace: renaming to %s was forbidden\n", to);
+			debug_printf("darwintrace: renaming to %s was forbidden\n", to);
 			errno = EACCES;
 			result = -1;
 		}
@@ -900,9 +933,42 @@ int stat(const char * path, struct stat * sb)
 #undef stat
 }
 
+#if defined(__DARWIN_64_BIT_INO_T) && !defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE)
+
+int stat64(const char * path, struct stat64 * sb)
+{
+#define stat64(path, sb) syscall(SYS_stat64, path, sb)
+	int result=0;
+	char newpath[260];
+		
+	*newpath=0;
+	if(!is_directory(path)&&__darwintrace_is_in_sandbox(path, newpath)==0)
+	{
+		errno=ENOENT;
+		result=-1;
+	}else
+	{
+		if(*newpath)
+			path=newpath;
+			
+		result=stat64(path, sb);
+	}
+	
+	return result;
+#undef stat64
+}
+
+int stat$INODE64(const char * path, struct stat64 * sb)
+{
+    return stat64(path, sb);
+}
+
+#endif /* defined(__DARWIN_64_BIT_INO_T) && !defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE) */
+
+
 int lstat(const char * path, struct stat * sb)
 {
-#define stat(path, sb) syscall(SYS_lstat, path, sb)
+#define lstat(path, sb) syscall(SYS_lstat, path, sb)
 	int result=0;
 	char newpath[260];
 	
@@ -916,9 +982,41 @@ int lstat(const char * path, struct stat * sb)
 		if(*newpath)
 			path=newpath;
 			
-		result=stat(path, sb);
+		result=lstat(path, sb);
 	}
 	
 	return result;
-#undef stat
-}
\ No newline at end of file
+#undef lstat
+}
+
+#if defined(__DARWIN_64_BIT_INO_T) && !defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE)
+
+int lstat64(const char * path, struct stat64 * sb)
+{
+#define lstat64(path, sb) syscall(SYS_lstat64, path, sb)
+	int result=0;
+	char newpath[260];
+	
+	*newpath=0;
+	if(!is_directory(path)&&__darwintrace_is_in_sandbox(path, newpath)==0)
+	{
+		errno=ENOENT;
+		result=-1;
+	}else
+	{
+		if(*newpath)
+			path=newpath;
+			
+		result=lstat64(path, sb);
+	}
+	
+	return result;
+#undef lstat64
+}
+
+int lstat$INODE64(const char * path, struct stat64 * sb)
+{
+    return lstat64(path, sb);
+}
+
+#endif /* defined(__DARWIN_64_BIT_INO_T) && !defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE) */
