@@ -1,6 +1,6 @@
 /*
  * tracelib.c
- * $Id: tracelib.c 53280 2009-07-02 11:08:06Z jmr@macports.org $
+ * $Id: tracelib.c 69989 2010-07-25 14:14:55Z jmr@macports.org $
  *
  * Copyright (c) 2007 Eugene Pimenov (GSoC), The MacPorts Project.
  * All rights reserved.
@@ -30,7 +30,10 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#if HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <string.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -47,6 +50,26 @@
 #include <limits.h>
 #include "tracelib.h"
 
+#ifndef HAVE_STRLCPY
+/* Define strlcpy if it's not available. */
+size_t strlcpy(char* dst, const char* src, size_t size);
+size_t strlcpy(char* dst, const char* src, size_t size)
+{
+	size_t result = strlen(src);
+	if (size > 0)
+	{
+		size_t copylen = size - 1;
+		if (copylen > result)
+		{
+			copylen = result;
+		}
+		memcpy(dst, src, copylen);
+		dst[copylen] = 0;
+	}
+	return result;
+}
+#endif
+
 static char * name;
 static char * sandbox;
 static char * filemap, * filemap_end;
@@ -158,7 +181,9 @@ static char process_line(int sock)
 		return 0;
 	if(!len)
 		return 0;
-	buf[len]=0;
+	
+	buf[len] = '\0';
+	
 	for(t=buf;*t&&t-buf<(int)sizeof(buf);t=next_t)
 	{
 		next_t = t+strlen(t)+1;
@@ -176,7 +201,7 @@ static char process_line(int sock)
 						ui_warn("recv failed");
 						return 0;
 					}
-					if(*end_of_t++ == 0)
+					if(*end_of_t++ == '\0')
 						break;
 				}
 			}
@@ -188,7 +213,11 @@ static char process_line(int sock)
 			ui_warn("malformed command %s", t);
 			break;
 		}
-		*f++=0;
+
+		/* Replace \t with \0 */
+		*f = '\0';
+		/* Advance pointer to arguments */
+		f++;
 
 		if(!strcmp(t, "filemap"))
 		{
@@ -243,13 +272,13 @@ static void send_file_map(int sock)
 			append_allow("/bin", 0);
 			append_allow("/sbin", 0);
 			append_allow("/dev", 0);
-			append_allow(Tcl_GetVar(interp, "macports::prefix", TCL_GLOBAL_ONLY), 2);
+			append_allow(Tcl_GetVar(interp, "prefix", TCL_GLOBAL_ONLY), 2);
 			/* If there is no SDK we will allow everything in /usr /System/Library etc, else add binaries to allow, and redirect root to SDK. */
 			if(sdk&&*sdk)
 			{
 				char buf[260];
 				buf[0] = '\0';
-				strlcat(buf, Tcl_GetVar(interp, "macports::developer_dir", TCL_GLOBAL_ONLY), 260);
+				strlcat(buf, Tcl_GetVar(interp, "developer_dir", TCL_GLOBAL_ONLY), 260);
 				strlcat(buf, "/SDKs/", 260);
 				strlcat(buf, sdk, 260);
 			
@@ -265,7 +294,7 @@ static void send_file_map(int sock)
 				append_allow("/usr", 0);
 				append_allow("/System/Library", 0);
 				append_allow("/Library", 0);
-				append_allow(Tcl_GetVar(interp, "macports::developer_dir", TCL_GLOBAL_ONLY), 0);
+				append_allow(Tcl_GetVar(interp, "developer_dir", TCL_GLOBAL_ONLY), 0);
 			}
 		}else
 			append_allow("/", 0);
@@ -336,9 +365,9 @@ static void ui_msg(const char * severity, const char * format, va_list va)
 {
 	char buf[1024], tclcmd[32];
 	
-	vsprintf(buf, format, va);
+	vsnprintf(buf, sizeof(buf), format, va);
 	
-	sprintf(tclcmd, "ui_%s $warn", severity);
+	snprintf(tclcmd, sizeof(tclcmd), "ui_%s $warn", severity);
 	
 	Tcl_SetVar(interp, "warn", buf, 0);
 	
@@ -365,12 +394,9 @@ static void ui_info(const char * format, ...)
 	va_end(va);
 }
 
-static int TracelibRunCmd(Tcl_Interp * in)
+static int TracelibOpenSocketCmd(Tcl_Interp * in)
 {
 	struct sockaddr_un sun;
-	fd_set fdr;
-	int i;
-	int max_fd, max_used, socks[MAX_SOCKETS];
 	struct rlimit rl;
 	
 	pthread_mutex_lock(&sock_mutex);
@@ -380,6 +406,13 @@ static int TracelibRunCmd(Tcl_Interp * in)
 		return 0;
 	}
 	sock=socket(AF_UNIX, SOCK_STREAM, 0);
+	if (sock == -1) {
+		Tcl_SetErrno(errno);
+		Tcl_ResetResult(interp);
+		Tcl_AppendResult(interp, "socket: ", (char *) Tcl_PosixError(interp), NULL);
+		pthread_mutex_unlock(&sock_mutex);
+		return TCL_ERROR;
+	}
 	pthread_mutex_unlock(&sock_mutex);
 	
 	interp=in;
@@ -397,13 +430,30 @@ static int TracelibRunCmd(Tcl_Interp * in)
 	
 	sun.sun_family=AF_UNIX;
 	strlcpy(sun.sun_path, name, sizeof(sun.sun_path));
-	if(bind(sock, (struct sockaddr*)&sun, sizeof(sun))==-1)
-	{
-		Tcl_SetResult(interp, "Cannot bind socket", TCL_STATIC);
+	if (bind(sock, (struct sockaddr*)&sun, sizeof(sun)) == -1) {
+		Tcl_SetErrno(errno);
+		Tcl_ResetResult(interp);
+		Tcl_AppendResult(interp, "bind: ", (char *) Tcl_PosixError(interp), NULL);
 		return TCL_ERROR;
 	}
 	
-	listen(sock, 5);
+	if (listen(sock, 5) == -1) {
+		Tcl_SetErrno(errno);
+		Tcl_ResetResult(interp);
+		Tcl_AppendResult(interp, "listen: ", (char *) Tcl_PosixError(interp), NULL);
+		return TCL_ERROR;
+	}
+
+        return TCL_OK;
+}
+
+
+static int TracelibRunCmd(Tcl_Interp * in UNUSED)
+{
+	int max_fd, max_used, socks[MAX_SOCKETS];
+	fd_set fdr;
+	int i;
+
 	max_used=0;
 	max_fd=sock;
 	
@@ -485,7 +535,7 @@ static int TracelibRunCmd(Tcl_Interp * in)
 	return TCL_OK;
 }
 
-static int TracelibCleanCmd(Tcl_Interp * interp)
+static int TracelibCleanCmd(Tcl_Interp * interp UNUSED)
 {
 	#define safe_free(x) do{free(x); x=0;}while(0);
 	cleanuping=1;
@@ -509,7 +559,6 @@ static int TracelibCleanCmd(Tcl_Interp * interp)
 	enable_fence=0;
 	#undef safe_free
 	cleanuping=0;
-	Tcl_Eval(interp, "registry::close_file_map");
 	return TCL_OK;
 }
 
@@ -565,10 +614,11 @@ static int TracelibEnableFence(Tcl_Interp * interp UNUSED)
 int TracelibCmd(ClientData clientData UNUSED, Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 {
 	int result=TCL_OK;
-	static const char * options[]={"setname", "run", "clean", "setsandbox", "closesocket", "setdeps", "enablefence", 0};
+	static const char * options[]={"setname", "opensocket", "run", "clean", "setsandbox", "closesocket", "setdeps", "enablefence", 0};
 	typedef enum 
 	{
 		kSetName,
+		kOpenSocket,
 		kRun,
 		kClean,
 		kSetSandbox,
@@ -593,6 +643,9 @@ int TracelibCmd(ClientData clientData UNUSED, Tcl_Interp* interp, int objc, Tcl_
 		case kSetName:
 			result=TracelibSetNameCmd(interp, objc, objv);
 			break;
+		case kOpenSocket:
+			result=TracelibOpenSocketCmd(interp);
+			break;
 		case kRun:
 			result=TracelibRunCmd(interp);
 			break;
