@@ -1,8 +1,9 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portfetch.tcl
-# $Id: portfetch.tcl 55219 2009-08-07 17:04:19Z raimue@macports.org $
+# $Id: portfetch.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
 #
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2004 - 2011 The MacPorts Project
+# Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,7 +14,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 #
@@ -31,6 +32,7 @@
 #
 
 package provide portfetch 1.0
+package require fetch_common 1.0
 package require portutil 1.0
 package require Pextlib 1.0
 
@@ -43,21 +45,13 @@ target_prerun ${org.macports.fetch} portfetch::fetch_start
 namespace eval portfetch {
     namespace export suffix
     variable fetch_urls {}
-    variable urlmap
-    array set urlmap {}
-}
-
-# Name space for internal site lists storage
-namespace eval portfetch::mirror_sites {
-    variable sites
-
-    array set sites {}
 }
 
 # define options: distname master_sites
-options master_sites patch_sites extract.suffix distfiles patchfiles use_zip use_bzip2 use_lzma use_7z use_dmg dist_subdir \
+options master_sites patch_sites extract.suffix distfiles patchfiles use_bzip2 use_lzma use_xz use_zip use_7z use_dmg dist_subdir \
     fetch.type fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert \
     master_sites.mirror_subdir patch_sites.mirror_subdir \
+    bzr.url bzr.revision \
     cvs.module cvs.root cvs.password cvs.date cvs.tag cvs.method \
     svn.url svn.revision svn.method \
     git.cmd git.url git.branch \
@@ -65,6 +59,7 @@ options master_sites patch_sites extract.suffix distfiles patchfiles use_zip use
 
 # XXX we use the command framework to buy us some useful features,
 # but this is not a user-modifiable command
+commands bzr
 commands cvs
 commands svn
 
@@ -72,6 +67,13 @@ commands svn
 default extract.suffix .tar.gz
 default fetch.type standard
 
+default bzr.cmd {[findBinary bzr $portutil::autoconf::bzr_path]}
+default bzr.dir {${workpath}}
+default bzr.revision {-1}
+default bzr.pre_args {"--builtin --no-aliases checkout --lightweight"}
+default bzr.args ""
+default bzr.post_args {"-r ${bzr.revision} ${bzr.url} ${worksrcdir}"}
+
 default cvs.cmd {[findBinary cvs $portutil::autoconf::cvs_path]}
 default cvs.password ""
 default cvs.dir {${workpath}}
@@ -84,15 +86,14 @@ default cvs.pre_args {"-z9 -f -d ${cvs.root}"}
 default cvs.args ""
 default cvs.post_args {"${cvs.module}"}
 
-# we want the svn port so we know --trust-server-cert will work
-default svn.cmd {${prefix}/bin/svn}
+default svn.cmd {[portfetch::find_svn_path]}
 default svn.dir {${workpath}}
 default svn.method {export}
 default svn.revision ""
 default svn.env {}
 default svn.pre_args {"--non-interactive --trust-server-cert"}
 default svn.args ""
-default svn.post_args {"${svn.url}"}
+default svn.post_args ""
 
 default git.cmd {[findBinary git $portutil::autoconf::git_path]}
 default git.dir {${workpath}}
@@ -127,6 +128,7 @@ option_deprecate svn.tag svn.revision
 # Option-executed procedures
 option_proc use_bzip2 portfetch::set_extract_type
 option_proc use_lzma  portfetch::set_extract_type
+option_proc use_xz    portfetch::set_extract_type
 option_proc use_zip   portfetch::set_extract_type
 option_proc use_7z    portfetch::set_extract_type
 option_proc use_dmg   portfetch::set_extract_type
@@ -142,7 +144,11 @@ proc portfetch::set_extract_type {option action args} {
             }
             use_lzma {
                 set extract.suffix .tar.lzma
-                depends_extract-append bin:lzma:lzmautils
+                depends_extract-append bin:lzma:xz
+            }
+            use_xz {
+                set extract.suffix .tar.xz
+                depends_extract-append bin:xz:xz
             }
             use_zip {
                 set extract.suffix .zip
@@ -160,13 +166,21 @@ proc portfetch::set_extract_type {option action args} {
 }
 
 proc portfetch::set_fetch_type {option action args} {
+    global os.platform os.major
     if {[string equal ${action} "set"]} {
         switch $args {
+            bzr {
+                depends_fetch-append bin:bzr:bzr
+            }
             cvs {
                 depends_fetch-append bin:cvs:cvs
             }
             svn {
-                depends_fetch-append port:subversion
+                if {${os.platform} == "darwin" && ${os.major} >= 10} {
+                    depends_fetch-append bin:svn:subversion
+                } else {
+                    depends_fetch-append port:subversion
+                }
             }
             git {
                 depends_fetch-append bin:git:git-core
@@ -178,13 +192,24 @@ proc portfetch::set_fetch_type {option action args} {
     }
 }
 
+proc portfetch::find_svn_path {args} {
+    global prefix os.platform os.major
+    # Snow Leopard is the first Mac OS X version to include a recent enough svn (1.6.x) to support the --trust-server-cert option.
+    if {${os.platform} == "darwin" && ${os.major} >= 10} {
+        return [findBinary svn $portutil::autoconf::svn_path]
+    } else {
+        return ${prefix}/bin/svn
+    }
+}
+
 set_ui_prefix
 
 
-# Given a distname, return a suffix based on the use_zip / use_bzip2 / use_dmg / extract.suffix options
+# Given a distname, return the distname with extract.suffix appended
 proc portfetch::suffix {distname} {
     global extract.suffix fetch.type
     switch -- "${fetch.type}" {
+        bzr         -
         cvs         -
         svn         -
         git         -
@@ -197,146 +222,6 @@ proc portfetch::suffix {distname} {
 # Portfiles, but should better go somewhere else
 namespace import portfetch::suffix
 
-# Given a site url and the name of the distfile, assemble url and
-# return it.
-proc portfetch::assemble_url {site distfile} {
-    if {[string index $site end] != "/"} {
-        return "${site}/${distfile}"
-    } else {
-        return "${site}${distfile}"
-    }
-}
-
-# For a given mirror site type, e.g. "gnu" or "x11", check to see if there's a
-# pre-registered set of sites, and if so, return them.
-proc portfetch::mirror_sites {mirrors tag subdir} {
-    global UI_PREFIX name porturl mirror_sites.listfile mirror_sites.listpath dist_subdir
-    global global_mirror_site fallback_mirror_site
-
-    set mirrorfile [getportresourcepath $porturl [file join ${mirror_sites.listpath} ${mirror_sites.listfile}]]
-    if {[file exists $mirrorfile]} {
-        source $mirrorfile
-    }
-
-    if {![info exists portfetch::mirror_sites::sites($mirrors)]} {
-        if {$mirrors != $global_mirror_site && $mirrors != $fallback_mirror_site} {
-            ui_warn "[format [msgcat::mc "No mirror sites on file for class %s"] $mirrors]"
-        }
-        return {}
-    }
-
-    set ret [list]
-    foreach element $portfetch::mirror_sites::sites($mirrors) {
-
-        # here we have the chance to take a look at tags, that possibly
-        # have been assigned in mirror_sites.tcl
-        set splitlist [split $element :]
-        # every element is a URL, so we'll always have multiple elements. no need to check
-        set element "[lindex $splitlist 0]:[lindex $splitlist 1]"
-        set mirror_tag "[lindex $splitlist 2]"
-
-        set name_re {\$(?:name\y|\{name\})}
-        # if the URL has $name embedded, kill any mirror_tag that may have been added
-        # since a mirror_tag and $name are incompatible
-        if {[regexp $name_re $element]} {
-            set mirror_tag ""
-        }
-
-        if {$mirror_tag == "mirror"} {
-            set thesubdir ${dist_subdir}
-        } elseif {$subdir == "" && $mirror_tag != "nosubdir"} {
-            set thesubdir ${name}
-        } else {
-            set thesubdir ${subdir}
-        }
-
-        # parse an embedded $name. if present, remove the subdir
-        if {[regsub $name_re $element $thesubdir element] > 0} {
-            set thesubdir ""
-        }
-
-        if {"$tag" != ""} {
-            eval append element "${thesubdir}:${tag}"
-        } else {
-            eval append element "${thesubdir}"
-        }
-
-        eval lappend ret $element
-    }
-
-    return $ret
-}
-
-# Checks sites.
-# sites tags create variables in the portfetch:: namespace containing all sites
-# within that tag distfiles are added in $site $distfile format, where $site is
-# the name of a variable in the portfetch:: namespace containing a list of fetch
-# sites
-proc portfetch::checksites {args} {
-    global patch_sites master_sites master_sites.mirror_subdir \
-        patch_sites.mirror_subdir fallback_mirror_site global_mirror_site env
-    variable urlmap
-
-    append master_sites " ${global_mirror_site} ${fallback_mirror_site}"
-    if {[info exists env(MASTER_SITE_LOCAL)]} {
-    set master_sites [concat $env(MASTER_SITE_LOCAL) $master_sites]
-    }
-
-    append patch_sites " ${global_mirror_site} ${fallback_mirror_site}"
-    if {[info exists env(PATCH_SITE_LOCAL)]} {
-    set patch_sites [concat $env(PATCH_SITE_LOCAL) $patch_sites]
-    }
-
-    foreach list {master_sites patch_sites} {
-        upvar #0 $list uplist
-        if {![info exists uplist]} {
-            continue
-        }
-
-        set site_list [list]
-        foreach site $uplist {
-            if {[regexp {([a-zA-Z]+://.+)} $site match site]} {
-                set site_list [concat $site_list $site]
-            } else {
-            set splitlist [split $site :]
-        if {[llength $splitlist] > 3 || [llength $splitlist] <1} {
-                    ui_error [format [msgcat::mc "Unable to process mirror sites for: %s, ignoring."] $site]
-        }
-        set mirrors "[lindex $splitlist 0]"
-        set subdir "[lindex $splitlist 1]"
-        set tag "[lindex $splitlist 2]"
-                if {[info exists $list.mirror_subdir]} {
-                    append subdir "[set ${list}.mirror_subdir]"
-                }
-                set site_list [concat $site_list [mirror_sites $mirrors $tag $subdir]]
-            }
-        }
-
-        # add in the global and fallback mirrors for each tag
-        foreach site $site_list {
-            if {[regexp {([a-zA-Z]+://.+/?):([0-9A-Za-z_-]+)$} $site match site tag]} {
-                if {![info exists extras_added($tag)]} {
-                    set site_list [concat $site_list [mirror_sites $global_mirror_site $tag ""] [mirror_sites $fallback_mirror_site $tag ""]]
-                    if {[string equal $list master_sites] && [info exists env(MASTER_SITE_LOCAL)]} {
-                        set site_list [concat [list $env(MASTER_SITE_LOCAL)] $site_list]
-                    } elseif {[string equal $list patch_sites] && [info exists env(PATCH_SITE_LOCAL)]} {
-                        set site_list [concat [list $env(PATCH_SITE_LOCAL)] $site_list]
-                    }
-                    set extras_added($tag) yes
-                }
-            }
-        }
-
-        foreach site $site_list {
-        if {[regexp {([a-zA-Z]+://.+/?):([0-9A-Za-z_-]+)$} $site match site tag]} {
-                lappend urlmap($tag) $site
-            } else {
-                lappend urlmap($list) $site
-            }
-        }
-    }
-}
-
 # Checks patch files and their tags to assemble url lists for later fetching
 proc portfetch::checkpatchfiles {urls} {
     global patchfiles all_dist_files patch_sites filespath
@@ -362,7 +247,7 @@ proc portfetch::checkpatchfiles {urls} {
 
 # Checks dist files and their tags to assemble url lists for later fetching
 proc portfetch::checkdistfiles {urls} {
-    global distfiles all_dist_files master_sites filespath
+    global distfiles all_dist_files filespath
     upvar $urls fetch_urls
 
     if {[info exists distfiles]} {
@@ -381,90 +266,37 @@ proc portfetch::checkdistfiles {urls} {
     }
 }
 
-# sorts fetch_urls in order of ping time
-proc portfetch::sortsites {urls} {
-    global fallback_mirror_site master_sites
-    upvar $urls fetch_urls
-    variable urlmap
-
-    set fallback_mirror_list [mirror_sites $fallback_mirror_site {} {}]
-
-    foreach {url_var distfile} $fetch_urls {
-        if {![info exists urlmap($url_var)]} {
-            ui_error [format [msgcat::mc "No defined site for tag: %s, using master_sites"] $url_var]
-            set urlmap($url_var) $master_sites
-        }
-        set urllist $urlmap($url_var)
-        set hosts {}
-        set hostregex {[a-zA-Z]+://([a-zA-Z0-9\.\-_]+)}
-
-        if {[llength $urllist] - [llength $fallback_mirror_list] <= 1} {
-            # there is only one mirror, no need to ping or sort
-            continue
-        }
-
-        foreach site $urllist {
-            regexp $hostregex $site -> host
-
-            if { [info exists seen($host)] } {
-                continue
-            }
-            foreach fallback $fallback_mirror_list {
-                if {[string match [append fallback *] $site]} {
-                    # don't bother pinging fallback mirrors
-                    set seen($host) yes
-                    # and make them sort to the very end of the list
-                    set pingtimes($host) 20000
-                    break
-                }
-            }
-            if { ![info exists seen($host)] } {
-                if {[catch {set fds($host) [open "|ping -noq -c3 -t3 $host | grep round-trip | cut -d / -f 5"]}]} {
-                    ui_debug "Spawning ping for $host failed"
-                    # will end up after all hosts that were pinged OK but before those that didn't respond
-                    set pingtimes($host) 5000
-                } else {
-                    ui_debug "Pinging $host..."
-                    set seen($host) yes
-                    lappend hosts $host
-                }
-            }
-        }
-
-        foreach host $hosts {
-            set len [gets $fds($host) pingtimes($host)]
-            if { [catch { close $fds($host) }] || ![string is double -strict $pingtimes($host)] } {
-                # ping failed, so put it last in the list (but before the fallback mirrors)
-                set pingtimes($host) 10000
-            }
-            ui_debug "$host ping time is $pingtimes($host)"
-        }
-
-        set pinglist {}
-        foreach site $urllist {
-            regexp $hostregex $site -> host
-            lappend pinglist [ list $site $pingtimes($host) ]
-        }
-
-        set pinglist [ lsort -real -index 1 $pinglist ]
-
-        set urlmap($url_var) {}
-        foreach pair $pinglist {
-            lappend urlmap($url_var) [lindex $pair 0]
-        }
-    }
+# returns full path to mirror list file
+proc portfetch::get_full_mirror_sites_path {} {
+    global mirror_sites.listfile mirror_sites.listpath porturl
+    return [getportresourcepath $porturl [file join ${mirror_sites.listpath} ${mirror_sites.listfile}]]
 }
 
 # Perform the full checksites/checkpatchfiles/checkdistfiles sequence.
 # This method is used by distcheck target.
 proc portfetch::checkfiles {urls} {
+    global global_mirror_site fallback_mirror_site
     upvar $urls fetch_urls
 
-    checksites
+    checksites [list patch_sites [list $global_mirror_site $fallback_mirror_site PATCH_SITE_LOCAL] \
+                master_sites [list $global_mirror_site $fallback_mirror_site MASTER_SITE_LOCAL]] \
+               [get_full_mirror_sites_path]
     checkpatchfiles fetch_urls
     checkdistfiles fetch_urls
 }
 
+# Perform a bzr fetch
+proc portfetch::bzrfetch {args} {
+    if {[catch {command_exec bzr "" "2>&1"} result]} {
+        return -code error [msgcat::mc "Bazaar checkout failed"]
+    }
+
+    if {[info exists patchfiles]} {
+        return [portfetch::fetchfiles]
+    }
+
+    return 0
+}
 
 # Perform a CVS login and fetch, storing the CVS login
 # information in a custom .cvspass file
@@ -514,13 +346,17 @@ proc portfetch::cvsfetch {args} {
 
 # Perform an svn fetch
 proc portfetch::svnfetch {args} {
-    global svn.args svn.revision svn.method
+    global svn.args svn.method svn.revision svn.url
 
-    set svn.args "${svn.method} ${svn.args}"
-    if {[string length ${svn.revision}]} {
-        set svn.args "${svn.args} -r ${svn.revision}"
+    if {[regexp {\s} ${svn.url}]} {
+        return -code error [msgcat::mc "Subversion URL cannot contain whitespace"]
     }
 
+    if {[string length ${svn.revision}]} {
+        append svn.url "@${svn.revision}"
+    }
+    set svn.args "${svn.method} ${svn.args} ${svn.url}"
+
     if {[catch {command_exec svn "" "2>&1"} result]} {
         return -code error [msgcat::mc "Subversion check out failed"]
     }
@@ -587,25 +423,10 @@ proc portfetch::hgfetch {args} {
 proc portfetch::fetchfiles {args} {
     global distpath all_dist_files UI_PREFIX
     global fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time
-    global distfile site
-    global portverbose
+    global fallback_mirror_site portverbose usealtworkpath altprefix
     variable fetch_urls
     variable urlmap
 
-    if {![file isdirectory $distpath]} {
-        if {[catch {file mkdir $distpath} result]} {
-            elevateToRoot "fetch"
-            set elevated yes
-            if {[catch {file mkdir $distpath} result]} {
-                return -code error [format [msgcat::mc "Unable to create distribution files path: %s"] $result]
-            }
-        }
-    }
-    chownAsRoot $distpath
-    if {[info exists elevated] && $elevated == yes} {
-        dropPrivileges
-    }
-
     set fetch_options {}
     if {[string length ${fetch.user}] || [string length ${fetch.password}]} {
         lappend fetch_options -u
@@ -626,46 +447,35 @@ proc portfetch::fetchfiles {args} {
     set sorted no
 
     foreach {url_var distfile} $fetch_urls {
-        if {![file isfile $distpath/$distfile]} {
+        if {![file isfile "${distpath}/${distfile}"]} {
             ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $distfile $distpath]"
             if {![file writable $distpath]} {
                 return -code error [format [msgcat::mc "%s must be writable"] $distpath]
             }
+            if {!$usealtworkpath && [file isfile ${altprefix}${distpath}/${distfile}]} {
+                if {[catch {file link -hard "${distpath}/${distfile}" "${altprefix}${distpath}/${distfile}"}]} {
+                    ui_debug "failed to hardlink ${distfile} into distpath, copying instead"
+                    file copy "${altprefix}${distpath}/${distfile}" "${distpath}/${distfile}"
+                }
+                ui_info "Found $distfile in ${altprefix}${distpath}"
+                continue
+            }
             if {!$sorted} {
-                sortsites fetch_urls
+                sortsites fetch_urls [mirror_sites $fallback_mirror_site {} {} [get_full_mirror_sites_path]] master_sites
                 set sorted yes
             }
             if {![info exists urlmap($url_var)]} {
                 ui_error [format [msgcat::mc "No defined site for tag: %s, using master_sites"] $url_var]
-                set urlmap($url_var) $master_sites
+                set urlmap($url_var) $urlmap(master_sites)
             }
             unset -nocomplain fetched
             foreach site $urlmap($url_var) {
-                ui_msg "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $distfile $site]"
+                ui_notice "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $distfile $site]"
                 set file_url [portfetch::assemble_url $site $distfile]
-                set effectiveURL ""
-                if {![catch {eval curl fetch --effective-url effectiveURL $fetch_options {$file_url} ${distpath}/${distfile}.TMP} result] &&
+                if {![catch {eval curl fetch $fetch_options {$file_url} {"${distpath}/${distfile}.TMP"}} result] &&
                     ![catch {file rename -force "${distpath}/${distfile}.TMP" "${distpath}/${distfile}"} result]} {
-
-                    # Special hack to check for sourceforge mirrors, which don't return a proper error code on failure
-                    if {![string equal $effectiveURL $file_url] &&
-                        [string match "http://*sourceforge.net/*" $file_url] &&
-                        [string match "http://*sourceforge.net/projects/*/files/" $effectiveURL]} {
-
-                        # *SourceForge hackage in effect*
-                        # The url seen by curl seems to have been a redirect to the sourceforge mirror page
-                        ui_debug "[msgcat::mc "Fetching from sourceforge mirror failed"]"
-                        file delete -force "${distpath}/${distfile}.TMP"
-
-                        # Continue on to try the next mirror, if any
-                    } else {
-
-                        # Successful fetch
-                        set fetched 1
-                        break
-
-                    }
-
+                    set fetched 1
+                    break
                 } else {
                     ui_debug "[msgcat::mc "Fetching failed:"]: $result"
                     file delete -force "${distpath}/${distfile}.TMP"
@@ -703,20 +513,30 @@ proc portfetch::fetch_addfilestomap {filemapname} {
 
 # Initialize fetch target and call checkfiles.
 proc portfetch::fetch_init {args} {
-    global distfiles distname distpath all_dist_files dist_subdir fetch.type fetch_init_done
     variable fetch_urls
 
-    if {[info exists distpath] && [info exists dist_subdir] && ![info exists fetch_init_done]} {
-        set distpath ${distpath}/${dist_subdir}
-        set fetch_init_done yes
-    }
     portfetch::checkfiles fetch_urls
 }
 
 proc portfetch::fetch_start {args} {
-    global UI_PREFIX name
+    global UI_PREFIX subport distpath
 
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Fetching %s"] $name]"
+    ui_notice "$UI_PREFIX [format [msgcat::mc "Fetching %s"] $subport]"
+
+    # create and chown $distpath
+    if {![file isdirectory $distpath]} {
+        if {[catch {file mkdir $distpath} result]} {
+            elevateToRoot "fetch"
+            set elevated yes
+            if {[catch {file mkdir $distpath} result]} {
+                return -code error [format [msgcat::mc "Unable to create distribution files path: %s"] $result]
+            }
+        }
+    }
+    chownAsRoot $distpath
+    if {[info exists elevated] && $elevated == yes} {
+        dropPrivileges
+    }
 }
 
 # Main fetch routine
@@ -724,7 +544,7 @@ proc portfetch::fetch_start {args} {
 # there are no files to download. Otherwise, either do a cvs checkout
 # or call the standard fetchfiles procedure
 proc portfetch::fetch_main {args} {
-    global distname distpath all_dist_files fetch.type
+    global all_dist_files fetch.type
 
     # Check for files, download if necessary
     if {![info exists all_dist_files] && "${fetch.type}" == "standard"} {
@@ -733,6 +553,7 @@ proc portfetch::fetch_main {args} {
 
     # Fetch the files
     switch -- "${fetch.type}" {
+        bzr     { return [bzrfetch] }
         cvs     { return [cvsfetch] }
         svn     { return [svnfetch] }
         git     { return [gitfetch] }
