@@ -1,9 +1,10 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portlivecheck.tcl
 #
-# $Id: portlivecheck.tcl 29930 2007-10-15 15:48:30Z jmpp@macports.org $
+# $Id: portlivecheck.tcl 80799 2011-07-17 18:33:07Z jmr@macports.org $
 #
-# Copyright (c) 2005-2006 Paul Guyot <pguyot@kallisys.net>,
+# Copyright (c) 2007-2011 The MacPorts Project
+# Copyright (c) 2005-2007 Paul Guyot <pguyot@kallisys.net>,
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -15,10 +16,10 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its
+# 3. Neither the name of The MacPorts Project nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -36,141 +37,129 @@ package provide portlivecheck 1.0
 package require portutil 1.0
 package require portfetch 1.0
 
-set org.macports.livecheck [target_new org.macports.livecheck livecheck_main]
+set org.macports.livecheck [target_new org.macports.livecheck portlivecheck::livecheck_main]
 target_runtype ${org.macports.livecheck} always
 target_state ${org.macports.livecheck} no
 target_provides ${org.macports.livecheck} livecheck
 target_requires ${org.macports.livecheck} main
 
+namespace eval portlivecheck {
+}
+
 # define options
-options livecheck.url livecheck.check livecheck.md5 livecheck.regex livecheck.name livecheck.distname livecheck.version
+options livecheck.url livecheck.type livecheck.check livecheck.md5 livecheck.regex livecheck.name livecheck.distname livecheck.version livecheck.ignore_sslcert
 
 # defaults
 default livecheck.url {$homepage}
 default livecheck.check default
+default livecheck.type default
 default livecheck.md5 ""
 default livecheck.regex ""
 default livecheck.name default
 default livecheck.distname default
 default livecheck.version {$version}
+default livecheck.ignore_sslcert yes
+
+# Deprecation
+option_deprecate livecheck.check livecheck.type
+
+proc portlivecheck::livecheck_main {args} {
+    global livecheck.url livecheck.type livecheck.md5 livecheck.regex livecheck.name livecheck.distname livecheck.version
+    global livecheck.ignore_sslcert
+    global homepage portpath workpath
+    global master_sites name subport distfiles
 
-proc livecheck_main {args} {
-    global livecheck.url livecheck.check livecheck.md5 livecheck.regex livecheck.name livecheck.distname livecheck.version
-    global homepage portname portpath workpath
-    global master_sites name distfiles
-    
     set updated 0
     set updated_version "unknown"
     set has_master_sites [info exists master_sites]
     set has_homepage [info exists homepage]
 
-    set tempfile ${workpath}/livecheck.TMP
+    set tempfile [mktemp "/tmp/mports.livecheck.XXXXXXXX"]
     set port_moddate [file mtime ${portpath}/Portfile]
 
     ui_debug "Portfile modification date is [clock format $port_moddate]"
     ui_debug "Port (livecheck) version is ${livecheck.version}"
 
-    # Determine the default type depending on the mirror.
-    if {${livecheck.check} eq "default"} {
+    set curl_options {}
+    if [tbool livecheck.ignore_sslcert] {
+        lappend curl_options "--ignore-ssl-cert"
+    }
+
+    # Check _resources/port1.0/livecheck for available types.
+    set types_dir [getdefaultportresourcepath "port1.0/livecheck"]
+    if {[catch {set available_types [glob -directory $types_dir -tails -types f *.tcl]} result]} {
+        return -code 1 "No available types were found. Check '$types_dir'."
+    }
+
+    # Convert available_types from a list of files (e.g., { freshmeat.tcl
+    # gnu.tcl ... }) into a string in the format "type|type|..." (e.g.,
+    # "freshmeat|gnu|...").
+    set available_types [regsub -all {\.tcl} [join $available_types |] {}]
+
+    if {${livecheck.type} eq "default"} {
+        # Determine the default type from the mirror.
         if {$has_master_sites} {
             foreach {master_site} ${master_sites} {
-                if {[regexp {^(sourceforge|freshmeat|googlecode|gnu)(?::([^:]+))?} ${master_site} _ site subdir]} {
+                if {[regexp "^($available_types)(?::(\[^:\]+))?" ${master_site} _ site subdir]} {
+                    set subdirs [split $subdir /]
+                    if {[llength $subdirs] > 1} {
+                        if {[lindex $subdirs 0] == "project"} {
+                            set subdir [lindex $subdirs 1]
+                        } else {
+                            set subdir ""
+                        }
+                    }
                     if {${subdir} ne "" && ${livecheck.name} eq "default"} {
                         set livecheck.name ${subdir}
                     }
-                    set livecheck.check ${site}
+                    set livecheck.type ${site}
 
                     break
                 }
             }
         }
-        if {${livecheck.check} eq "default"} {
-            set livecheck.check "freshmeat"
+        # If the default type cannot be determined from the mirror, use the
+        # fallback.
+        if {${livecheck.type} eq "default"} {
+            set livecheck.type "fallback"
         }
         if {$has_homepage} {
             if {[regexp {^http://code.google.com/p/([^/]+)} $homepage _ tag]} {
-                if {${livecheck.name} eq "default"} {
-                    set livecheck.name $tag
-                }
-                set livecheck.check "googlecode"
+                set livecheck.type "googlecode"
             } elseif {[regexp {^http://www.gnu.org/software/([^/]+)} $homepage _ tag]} {
-                if {${livecheck.name} eq "default"} {
-                    set livecheck.name $tag
-                }
-                set livecheck.check "gnu"
+                set livecheck.type "gnu"
             }
         }
     }
-    if {${livecheck.name} eq "default"} {
-        set livecheck.name $name
+    if {[lsearch -exact [split $available_types "|"] ${livecheck.type}] != -1} {
+        # Load the defaults from _resources/port1.0/livecheck/${livecheck.type}.tcl.
+        set defaults_file "$types_dir/${livecheck.type}.tcl"
+        ui_debug "Loading the defaults from '$defaults_file'"
+        if {[catch {source $defaults_file} result]} {
+            global errorInfo
+            ui_debug "$errorInfo: result"
+            return -code 1 "The defaults could not be loaded from '$defaults_file'."
+        }
     }
 
-    # Perform the check depending on the type.
-    switch ${livecheck.check} {
-        "freshmeat" {
-            if {!$has_homepage || ${livecheck.url} eq ${homepage}} {
-                set livecheck.url "http://freshmeat.net/projects-xml/${livecheck.name}/${livecheck.name}.xml"
-            }
-            if {${livecheck.regex} eq ""} {
-                set livecheck.regex "<latest_release_version>(.*)</latest_release_version>"
-            }
-            set livecheck.check "regex"
-        }
-        "sourceforge" {
-            if {!$has_homepage || ${livecheck.url} eq ${homepage}} {
-                set livecheck.url "http://sourceforge.net/export/rss2_projfiles.php?project=${livecheck.name}"
-            }
-            if {${livecheck.distname} eq "default"} {
-                set livecheck.distname ${livecheck.name}
-            }
-            if {${livecheck.regex} eq ""} {
-                set livecheck.regex "<title>${livecheck.distname} (.*) released.*</title>"
-            }
-            set livecheck.check "regex"
-        }
-        "googlecode" {
-            if {!$has_homepage || ${livecheck.url} eq ${homepage}} {
-                set livecheck.url "http://code.google.com/p/${livecheck.name}/downloads/list"
-            }
-            if {${livecheck.distname} eq "default"} {
-                set livecheck.distname [regsub ***=${livecheck.version} [file tail [lindex ${distfiles} 0]] (.*)]
-            }
-            if {${livecheck.regex} eq ""} {
-                set livecheck.regex {<a href="http://${livecheck.name}.googlecode.com/files/${livecheck.distname}"}
-            }
-            set livecheck.check "regex"
-        }
-        "gnu" {
-            if {!$has_homepage || ${livecheck.url} eq ${homepage}} {
-                set livecheck.url "http://ftp.gnu.org/gnu/${livecheck.name}/?C=M&O=D"
-            }
-            if {${livecheck.distname} eq "default"} {
-                set livecheck.distname ${livecheck.name}
-            }
-            if {${livecheck.regex} eq ""} {
-                set livecheck.regex {${livecheck.distname}-(\\d+(?:\\.\\d+)*)}
-            }
-            set livecheck.check "regex"
-        }
-    }
-    
     # de-escape livecheck.url
     set livecheck.url [join ${livecheck.url}]
-    
-    switch ${livecheck.check} {
+
+    switch ${livecheck.type} {
         "regex" -
         "regexm" {
             # single and multiline regex
             ui_debug "Fetching ${livecheck.url}"
-            if {[catch {curl fetch ${livecheck.url} $tempfile} error]} {
-                ui_error "cannot check if $portname was updated ($error)"
+            if {[catch {eval curl fetch $curl_options {${livecheck.url}} $tempfile} error]} {
+                ui_error "cannot check if $subport was updated ($error)"
+                set updated -1
             } else {
                 # let's extract the version from the file.
                 set chan [open $tempfile "r"]
                 set updated -1
-                set the_re [subst -nocommands -nobackslashes [join ${livecheck.regex}]]
-                ui_debug "The regex is >$the_re<"
-                if {${livecheck.check} == "regexm"} {
+                set the_re [join ${livecheck.regex}]
+                ui_debug "The regex is \"$the_re\""
+                if {${livecheck.type} == "regexm"} {
                     set data [read $chan]
                     if {[regexp $the_re $data matched updated_version]} {
                         if {$updated_version != ${livecheck.version}} {
@@ -178,34 +167,40 @@ proc livecheck_main {args} {
                         } else {
                             set updated 0
                         }
-                        ui_debug "The regex matched >$matched<"
+                        ui_debug "The regex matched \"$matched\", extracted \"$updated_version\""
                     }
                 } else {
-                    while {1} {
-                        if {[gets $chan line] < 0} {
-                            break
-                        }
-                        if {[regexp $the_re $line matched updated_version]} {
-                            if {$updated_version != ${livecheck.version}} {
-                                set updated 1
-                            } else {
-                                set updated 0
+                    set updated_version 0
+                    set foundmatch 0
+                    while {[gets $chan line] >= 0} {
+                        if {[regexp $the_re $line matched upver]} {
+                            set foundmatch 1
+                            if {$updated_version == 0 || [rpm-vercomp $upver $updated_version] > 0} {
+                                set updated_version $upver
                             }
-                            ui_debug "The regex matched >$matched<"
-                            break
+                            ui_debug "The regex matched \"$matched\", extracted \"$upver\""
+                        }
+                    }
+                    if {$foundmatch == 1} {
+                        if {$updated_version == 0} {
+                            set updated -1
+                        } elseif {$updated_version != ${livecheck.version}} {
+                            set updated 1
+                        } else {
+                            set updated 0
                         }
                     }
                 }
                 close $chan
                 if {$updated < 0} {
-                    ui_error "cannot check if $portname was updated (regex didn't match)"
+                    ui_error "cannot check if $subport was updated (regex didn't match)"
                 }
             }
         }
         "md5" {
             ui_debug "Fetching ${livecheck.url}"
-            if {[catch {curl fetch ${livecheck.url} $tempfile} error]} {
-                ui_error "cannot check if $portname was updated ($error)"
+            if {[catch {eval curl fetch $curl_options {${livecheck.url}} $tempfile} error]} {
+                ui_error "cannot check if $subport was updated ($error)"
                 set updated -1
             } else {
                 # let's compute the md5 sum.
@@ -219,7 +214,7 @@ proc livecheck_main {args} {
         "moddate" {
             set port_moddate [file mtime ${portpath}/Portfile]
             if {[catch {set updated [curl isnewer ${livecheck.url} $port_moddate]} error]} {
-                ui_error "cannot check if $portname was updated ($error)"
+                ui_error "cannot check if $subport was updated ($error)"
                 set updated -1
             } else {
                 if {!$updated} {
@@ -230,17 +225,17 @@ proc livecheck_main {args} {
         "none" {
         }
         default {
-            ui_error "unknown livecheck.check ${livecheck.check}"
+            ui_error "unknown livecheck.type ${livecheck.type}"
         }
     }
 
     file delete -force $tempfile
 
-    if {${livecheck.check} != "none"} {
+    if {${livecheck.type} != "none"} {
         if {$updated > 0} {
-            ui_msg "$portname seems to have been updated (port version: ${livecheck.version}, new version: $updated_version)"
+            ui_msg "$subport seems to have been updated (port version: ${livecheck.version}, new version: $updated_version)"
         } elseif {$updated == 0} {
-            ui_info "$portname seems to be up to date"
+            ui_info "$subport seems to be up to date"
         }
     }
 }
