@@ -1,9 +1,10 @@
 # receipt_flat.tcl
-# $Id: receipt_flat.tcl 30972 2007-11-12 17:11:33Z jmpp@macports.org $
+# $Id: receipt_flat.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
 #
+# Copyright (c) 2005, 2007-2009 The MacPorts Project
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
 # Copyright (c) 2004 Paul Guyot, The MacPorts Project.
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -43,6 +44,8 @@ namespace eval receipt_flat {
 
 # receipt_lastref is the last attributed index of receipts.
 variable receipt_lastref -1
+# maps port name,version,variants to the corresponding receipt ref
+variable ref_index
 
 ##
 #
@@ -72,7 +75,7 @@ proc get_head_entry_receipt_path {portname portversion} {
 
     # regex match case
     if {$portversion == 0} {
-	set x [glob -nocomplain [file join ${macports::registry.path} receipts ${portname}-*]]
+	set x [glob -nocomplain -directory [file join ${macports::registry.path} receipts] -- ${portname}-*]
 	if {[string length $x]} {
 	    set matchfile [lindex $x 0]
 		# Remove trailing .bz2, if any.
@@ -98,9 +101,14 @@ proc get_head_entry_receipt_path {portname portversion} {
 ##
 #
 # Open an existing entry and return its reference number.
-proc open_entry {name {version 0} {revision 0} {variants ""}} {
-	global macports::registry.installtype
+proc open_entry {name {version ""} {revision 0} {variants ""} {epoch ""}} {
 	global macports::registry.path
+	variable ref_index
+	
+	# if this entry is already open, just return the reference
+	if {[info exists ref_index($name,$version,$revision,$variants)]} {
+	    return $ref_index($name,$version,$revision,$variants)
+	}
 
 	set receipt_path [file join ${macports::registry.path} receipts ${name}]
 
@@ -110,7 +118,7 @@ proc open_entry {name {version 0} {revision 0} {variants ""}} {
 		set receipt_file [get_head_entry_receipt_path $name $version]
 		
 		if {![string length $receipt_file]} {
-			if { $version != 0 } {
+			if { $version != "" } {
 				return -code error "Registry error: ${name} @${version}_${revision}${variants} not registered as installed."
 			} else {
 				return -code error "Registry error: ${name} not registered as installed."
@@ -118,17 +126,16 @@ proc open_entry {name {version 0} {revision 0} {variants ""}} {
 		}
 		
 		# Extract the version from the path.
-		if { $version == 0 } {
+		if { $version == "" } {
 			set theFileName [file tail $receipt_file]
 			regexp "^$name-(.*)\$" $theFileName match version
 		}
 	} else {
-		# If version wasn't specified, find out the version number.  This will
-		# depend on which installtype mode we're in, "direct" or "image"	
-		if { $version == 0 } {
-			# xxx: If we're in image mode, we really should have had the 
+		# If version wasn't specified, find out the version number.
+		if { $version == "" } {
+			# xxx: We really should have had the 
 			# version given to us.  How should we handle this?
-			set x [glob -nocomplain [file join ${receipt_path} *]]
+			set x [glob -nocomplain -directory ${receipt_path} *]
 			if { [string length $x] } {
 				set v [lindex [file split [lindex $x 0]] end]
 				regexp {([-_a-zA-Z0-9\.]+)_([0-9]*)([+-_a-zA-Z0-9]*)$} $v match version revision variants
@@ -154,7 +161,7 @@ proc open_entry {name {version 0} {revision 0} {variants ""}} {
 		set receipt_contents [read $receipt_handle]
 		close $receipt_handle
 	} else {
-		return -code error "Registry error: receipt for ${name} @${version}_${revision}${variants} seems to be compressed, but bzip2 couln't be found."
+		return -code error "Registry error: receipt for ${name} @${version}_${revision}${variants} seems to be compressed, but bzip2 couldn't be found."
 	}
 
 	set ref [new_entry]
@@ -172,7 +179,7 @@ proc open_entry {name {version 0} {revision 0} {variants ""}} {
 		# move the old receipt
 		set convertedDirPath [file join ${macports::registry.path} receipts_converted]
 		file mkdir $convertedDirPath
-		file rename $receipt_file $convertedDirPath
+		file rename -- $receipt_file $convertedDirPath
 	} elseif {[string match "# Version: *" $receipt_contents]} {
 		# This is new format
 		if {![string match "# Version: 1.0*" $receipt_contents]} {
@@ -181,7 +188,11 @@ proc open_entry {name {version 0} {revision 0} {variants ""}} {
 
 		# Remove any line starting with #
 		while {[regexp "(^|\n)#.*\n(.*)\$" $receipt_contents match foo receipt_contents]} {}
-		array set receipt_$ref $receipt_contents
+		if {[catch {array set receipt_$ref $receipt_contents} rcpterr]} {
+			ui_error "Malformed receipt for ${name} @${version}_${revision}${variants}"
+			ui_error "receipt_contents = '$receipt_contents'"
+			error $rcpterr
+		}
 	} else {
 		# This is old Images format
 
@@ -191,6 +202,8 @@ proc open_entry {name {version 0} {revision 0} {variants ""}} {
 		}
 	}
 	
+	set ref_index($name,$version,$revision,$variants) $ref
+	
 	return $ref
 }
 
@@ -282,6 +295,7 @@ proc convert_entry_from_HEAD {name version revision variants receipt_contents re
 
 	# Save the file_map afterwards
 	write_file_map
+	close_file_map
 	
 	# Save the entry to new format.
 	write_entry $ref $name $version $revision $variants
@@ -296,7 +310,6 @@ proc convert_entry_from_HEAD {name version revision variants receipt_contents re
 # version			the version of the port.
 # variants			the variants of the port.
 proc write_entry {ref name version {revision 0} {variants ""}} {
-	global macports::registry.installtype
 	variable receipt_$ref
 
 	set receipt_contents [array get receipt_$ref]
@@ -314,12 +327,12 @@ proc write_entry {ref name version {revision 0} {variants ""}} {
 	close $receipt_handle
 
 	if { [file exists ${receipt_file}] } {
-		system "rm -rf ${receipt_file}"
+		file delete -force -- "${receipt_file}"
 	} elseif { [file exists ${receipt_file}.bz2] } {
-		system "rm -rf ${receipt_file}.bz2"
+		file delete -force -- "${receipt_file}.bz2"
 	}
 
-	system "mv ${receipt_file}.tmp ${receipt_file}"
+	file rename -force -- "${receipt_file}.tmp" "${receipt_file}"
 
 	if { [file exists ${receipt_file}] && [file exists ${registry::autoconf::bzip2_path}] && ![info exists registry.nobzip] } {
 		system "${registry::autoconf::bzip2_path} -f ${receipt_file}"
@@ -331,9 +344,6 @@ proc write_entry {ref name version {revision 0} {variants ""}} {
 # Check to see if an entry exists
 proc entry_exists {name version {revision 0} {variants ""}} {
 	global macports::registry.path
-	variable receipt_handle 
-	variable receipt_file 
-	variable receipt_path
 
 	set receipt_path [file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
 	set receipt_file [file join ${receipt_path} receipt]
@@ -347,6 +357,19 @@ proc entry_exists {name version {revision 0} {variants ""}} {
 	return 0
 }
 
+# Check to see if any entry exists for the given port name
+proc entry_exists_for_name {name} {
+	global macports::registry.path
+
+	set receipt_path [file join ${macports::registry.path} receipts ${name}]
+
+	if {[llength [glob -nocomplain -directory $receipt_path */receipt{,.bz2}]] > 0} {
+		return 1
+	}
+
+	return 0
+}
+
 ##
 #
 # Store a property to a receipt that was loaded in memory.
@@ -384,19 +407,28 @@ proc property_retrieve {ref property} {
 # Delete an entry
 proc delete_entry {name version {revision 0} {variants ""}} {
 	global macports::registry.path
+	variable ref_index
+	
+	# if the entry is loaded, purge it
+	if {[info exists ref_index($name,$version,$revision,$variants)]} {
+	    set ref $ref_index($name,$version,$revision,$variants)
+	    variable receipt_${ref}
+	    array unset receipt_${ref}
+	    array unset ref_index "$name,$version,$revision,$variants"
+	}
 
 	set receipt_path [file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
 	if { [file exists ${receipt_path}] } {
 		# remove port receipt directory
 		ui_debug "deleting directory: ${receipt_path}"
-		file delete -force ${receipt_path}
+		file delete -force -- ${receipt_path}
 		# remove port receipt parent directory (if empty)
 		set receipt_dir [file join ${macports::registry.path} receipts ${name}]
 		if { [file isdirectory ${receipt_dir}] } {
 			# 0 item means empty.
 			if { [llength [readdir ${receipt_dir}]] == 0 } {
 				ui_debug "deleting directory: ${receipt_dir}"
-				file delete -force ${receipt_dir}
+				file delete -force -- ${receipt_dir}
 			} else {
 				ui_debug "${receipt_dir} is not empty"
 			}
@@ -426,6 +458,19 @@ proc installed {{name ""} {version ""}} {
 		}
 		# [PG] Huh?
 	} else {
+	    # We want to be case-insensitive but case-preserving, so the name gets
+	    # returned with the correct case even if it's wrong when given. To get the
+	    # correct case on a case-insensitive FS, we have to list the directory and
+	    # compare against each entry.
+	    set name_path [file join ${query_path} *]
+	    set name_entries [glob -nocomplain -types d ${name_path}]
+	    foreach entry $name_entries {
+	        set basename [file tail $entry]
+	        if {[string equal -nocase $basename $name]} {
+	            set name $basename
+	            break
+	        }
+	    }
 		set query_path [file join ${query_path} ${name}]
 		if { $version != "" } {
 			set query_path [file join ${query_path} ${version}]
@@ -437,6 +482,9 @@ proc installed {{name ""} {version ""}} {
 	set x [glob -nocomplain -types d ${query_path}]
 	set rlist [list]
 	foreach p $x {
+		if {![file isfile [file join $p receipt.bz2]] && ![file isfile [file join $p receipt]]} {
+			continue
+		}
 		set plist [list]
 		regexp {([-_a-zA-Z0-9\.]+)_([0-9]*)([+-_a-zA-Z0-9]*)$} [lindex [file split $p] end] match version revision variants
 		lappend plist [lindex [file split $p] end-1]
@@ -677,6 +725,19 @@ proc write_file_map {args} {
 	return 1
 }
 
+##
+# close the file map.
+# important to do this so the lock is released.
+# do not do anything if the file map wasn't open.
+#
+proc close_file_map {args} {
+	variable file_map
+
+	if { [info exists file_map] } {
+		filemap close file_map
+	}
+}
+
 # Dependency Map Code
 proc open_dep_map {args} {
 	global macports::registry.path
@@ -701,7 +762,7 @@ proc open_dep_map {args} {
 }
 
 # List all ports this one depends on
-proc list_depends {name} {
+proc list_depends {name {version ""} {revision ""} {variants 0}} {
 	variable dep_map
 	if { [llength $dep_map] < 1 && [info exists dep_map] } {
 		open_dep_map
@@ -716,7 +777,7 @@ proc list_depends {name} {
 }
 
 # List all the ports that depend on this port
-proc list_dependents {name} {
+proc list_dependents {name version revision variants} {
 	variable dep_map
 	if { [llength $dep_map] < 1 && [info exists dep_map] } {
 		open_dep_map
@@ -732,7 +793,11 @@ proc list_dependents {name} {
 
 proc register_dep {dep type port} {
 	variable dep_map
-	lappend dep_map [list $dep $type $port]
+	set newdep [list $dep $type $port]
+	# slow, but avoids duplicate entries building up
+	if {[lsearch -exact $dep_map $newdep] == -1} {
+	    lappend dep_map $newdep
+	}
 }
 
 proc unregister_dep {dep type port} {
@@ -746,6 +811,28 @@ proc unregister_dep {dep type port} {
 	set dep_map $new_map
 }
 
+# remove duplicate entries from the dep_map
+# (could be put there by older versions of MacPorts)
+proc clean_dep_map {args} {
+    variable dep_map
+    set new_map [list]
+    set oldlen [llength $dep_map]
+    ui_debug "Current dep_map has $oldlen entries"
+    foreach de $dep_map {
+        if {[lsearch -exact $new_map $de] == -1} {
+            lappend new_map $de
+        } else {
+            ui_debug "Removing $de from dep_map"
+        }
+    }
+    set dep_map $new_map
+    
+    set newlen [llength $dep_map]
+    set diff [expr $oldlen - $newlen]
+    ui_debug "New dep_map has $newlen entries"
+    ui_info "Removed $diff duplicate entries from the dependency map"
+}
+
 proc write_dep_map {args} {
 	global macports::registry.path
 	variable dep_map
@@ -759,9 +846,9 @@ proc write_dep_map {args} {
 	close $map_handle
 
     # don't both checking for presence, file delete doesn't error if file doesn't exist
-    file delete ${map_file} ${map_file}.bz2
+    file delete -- ${map_file} ${map_file}.bz2
 
-    file rename ${map_file}.tmp ${map_file}
+    file rename -- ${map_file}.tmp ${map_file}
 
 	if { [file exists ${map_file}] && [file exists ${registry::autoconf::bzip2_path}] && ![info exists registry.nobzip] } {
 		system "${registry::autoconf::bzip2_path} -f ${map_file}"
